@@ -43,13 +43,13 @@ class AsyncS3Bucket:
         <https://docs.aws.amazon.com/AmazonS3/latest/dev/UsingBucket.html>
         for details. In particular note that bucket names must be globally
         unique across all AWS accounts.
-    create : `bool` (optional)
+    create : `bool`, optional
         If True and the bucket does not exist, create it.
         If False then assume the bucket exists.
         You will typically want true if using a mock server (``domock`` true).
-    profile : `str` (optional)
+    profile : `str`, optional
         Profile name; use the default profile if None.
-    domock : `bool` (optional)
+    domock : `bool`, optional
         If True then start a mock S3 server.
         This is recommended for running in simulation mode.
 
@@ -102,16 +102,6 @@ class AsyncS3Bucket:
         self.mock = moto.mock_s3()
         self.mock.start()
 
-        # Set s3 authentication environment variables to bogus values
-        # to avoid any danger of writing to a real s3 server.
-        for env_var_name in (
-            "AWS_ACCESS_KEY_ID",
-            "AWS_SECRET_ACCESS_KEY",
-            "AWS_SECURITY_TOKEN",
-            "AWS_SESSION_TOKEN",
-        ):
-            os.environ[env_var_name] = "testing"
-
     def stop_mock(self):
         """Stop the mock s3 service, if running. A no-op if not running.
         """
@@ -135,7 +125,7 @@ class AsyncS3Bucket:
             or "ATBlue" for the AT Blue Fiber Spectrograph.
         s3instance : `str`
             S3 server instance. Typically "Summit", "Tucson" or "NCSA".
-        s3category : `str` (optional)
+        s3category : `str`, optional
             Category of S3 server. The default is "LFA",
             for the Large File Annex.
 
@@ -194,12 +184,12 @@ class AsyncS3Bucket:
             Dataset type.
         date : `astropy.time.Time`
             A date -- typically the date the data was taken.
-        other : `str` or `None` (optional)
+        other : `str` or `None`, optional
             Additional text to identify the data and make the key unique.
             If `None` use ``date.tai.isot``: ``date`` as TAI,
             in ISO-8601 format, with a "T" between the date and time
             and a precision of milliseconds.
-        suffix : `str` (optional)
+        suffix : `str`, optional
             Key suffix, e.g. ".fits".
 
         Returns
@@ -250,7 +240,7 @@ class AsyncS3Bucket:
             File-like object that can be read as *binary* data.
         key : `str`
             Name to use for the file in the bucket.
-        callback : callable (optional)
+        callback : callable, optional
             Function to call with updates while writing. The function receives
             one argument: the number of bytes written. If the transfer is
             successful then it will always be called at least once,
@@ -267,7 +257,7 @@ class AsyncS3Bucket:
             hdulist.writeto(fileobj)
             fileobj.seek(0)
         """
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         await loop.run_in_executor(None, self._sync_upload, fileobj, key, callback)
 
     async def download(self, key, callback=None):
@@ -277,7 +267,7 @@ class AsyncS3Bucket:
         ----------
         key : `str`
             Name of the file in the bucket.
-        callback : callable (optional)
+        callback : callable, optional
             Function to call with updates while writing. The function receives
             one argument: the number of bytes read so far. If the transfer is
             successful then it will always be called at least once,
@@ -298,7 +288,7 @@ class AsyncS3Bucket:
 
         The callback function is called by ``S3.Bucket.download_fileobj``.
         """
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         return await loop.run_in_executor(None, self._sync_download, key, callback)
 
     async def exists(self, key):
@@ -309,7 +299,7 @@ class AsyncS3Bucket:
         key : `str`
             Name of the potential file in the bucket.
         """
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         try:
             await loop.run_in_executor(None, self._sync_size, key)
         except botocore.exceptions.ClientError as e:
@@ -327,7 +317,7 @@ class AsyncS3Bucket:
         key : `str`
             Name of the file in the bucket.
         """
-        loop = asyncio.get_event_loop()
+        loop = asyncio.get_running_loop()
         return await loop.run_in_executor(None, self._sync_size, key)
 
     def _sync_upload(self, fileobj, key, callback):
