@@ -23,19 +23,21 @@ __all__ = ["BaseCscTestCase"]
 import abc
 import asyncio
 import contextlib
+import enum
 import logging
 import shutil
+import subprocess
 
 from . import base
 from . import sal_enums
 from . import testutils
 from .domain import Domain
 from .remote import Remote
+from .csc_utils import get_expected_summary_states
 
-# Timeout for fast operations (seconds)
-STD_TIMEOUT = 10
-# Timeout for slow operations, such as CSCs starting (seconds).
-LONG_TIMEOUT = 60
+# Standard timeout (sec)
+# Long to avoid unnecessary timeouts on slow CI systems.
+STD_TIMEOUT = 60
 
 
 class BaseCscTestCase(metaclass=abc.ABCMeta):
@@ -60,10 +62,10 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
     def basic_make_csc(self, initial_state, config_dir, simulation_mode, **kwargs):
         """Make and return a CSC.
 
-        initial_state : `lsst.ts.salobj.State` or `int` (optional)
-            The initial state of the CSC. Ignored except in simulation mode
-            because in normal operation the initial state is the current state
-            of the controller.
+        Parameters
+        ----------
+        initial_state : `lsst.ts.salobj.State` or `int`
+            The initial state of the CSC.
         config_dir : `str`
             Directory of configuration files, or None for the standard
             configuration directory (obtained from
@@ -81,71 +83,91 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
     @contextlib.asynccontextmanager
     async def make_csc(
         self,
-        initial_state,
+        initial_state=sal_enums.State.STANDBY,
         config_dir=None,
         simulation_mode=0,
         log_level=None,
-        timeout=LONG_TIMEOUT,
+        timeout=STD_TIMEOUT,
         **kwargs,
     ):
-        """Create a CSC and remote and wait for them to start.
+        """Create a CSC and remote and wait for them to start,
+        after setting a random $LSST_DDS_PARTITION_PREFIX.
 
         The csc is accessed as ``self.csc`` and the remote as ``self.remote``.
 
+        Reads and checks all but the last ``summaryState`` event during
+        startup.
+
         Parameters
         ----------
         name : `str`
             Name of SAL component.
-        index : `int` or `None` (optional)
-            SAL component index, or 0 or None if the component is not indexed.
-            A value is required if the component is indexed.
-        initial_state : `lsst.ts.salobj.State` or `int` (optional)
-            The initial state of the CSC.
-        config_dir : `str` (optional)
-            Directory of configuration files, or None for the standard
-            configuration directory (obtained from
+        initial_state : `lsst.ts.salobj.State` or `int`, optional
+            The initial state of the CSC. Defaults to STANDBY.
+        config_dir : `str`, optional
+            Directory of configuration files, or `None` (the default)
+            for the standard configuration directory (obtained from
             `ConfigureCsc._get_default_config_dir`).
-        simulation_mode : `int` (optional)
-            Simulation mode.
-        log_level : `int` or `None` (optional)
+        simulation_mode : `int`, optional
+            Simulation mode. Defaults to 0 because not all CSCs support
+            simulation. However, tests of CSCs that support simulation
+            will almost certainly want to set this nonzero.
+        log_level : `int` or `None`, optional
             Logging level, such as `logging.INFO`.
-            If None then do not set the log level, leaving the default
+            If `None` then do not set the log level, leaving the default
             behavior of `SalInfo`: increase the log level to INFO.
-        timeout : `float`
+        timeout : `float`, optional
             Time limit for the CSC to start (seconds).
-        **kwargs : `dict`
+        **kwargs : `dict`, optional
             Extra keyword arguments for `basic_make_csc`.
+            For a configurable CSC this may include ``settings_to_apply``,
+            especially if ``initial_state`` is DISABLED or ENABLED.
 
         Notes
         -----
-        Adds a logging.StreamHandler if one is not alread present.
+        Adds a logging.StreamHandler if one is not already present.
         """
-        testutils.set_random_lsst_dds_domain()
-        self.csc = self.basic_make_csc(
-            initial_state=initial_state,
-            config_dir=config_dir,
-            simulation_mode=simulation_mode,
-            **kwargs,
-        )
-        if len(self.csc.log.handlers) < 2:
-            self.csc.log.addHandler(logging.StreamHandler())
-        self.remote = Remote(
-            domain=self.csc.domain,
-            name=self.csc.salinfo.name,
-            index=self.csc.salinfo.index,
-        )
-        if log_level is not None:
-            self.csc.log.setLevel(log_level)
-
-        await asyncio.wait_for(
-            asyncio.gather(self.csc.start_task, self.remote.start_task),
-            timeout=timeout,
-        )
+        testutils.set_random_lsst_dds_partition_prefix()
+        self.remote = None
+        self.csc = None
         try:
+            self.csc = self.basic_make_csc(
+                initial_state=initial_state,
+                config_dir=config_dir,
+                simulation_mode=simulation_mode,
+                **kwargs,
+            )
+            if len(self.csc.log.handlers) < 2:
+                self.csc.log.addHandler(logging.StreamHandler())
+            self.remote = Remote(
+                domain=self.csc.domain,
+                name=self.csc.salinfo.name,
+                index=self.csc.salinfo.index,
+            )
+            if log_level is not None:
+                self.csc.log.setLevel(log_level)
+
+            await asyncio.wait_for(
+                asyncio.gather(self.csc.start_task, self.remote.start_task),
+                timeout=timeout,
+            )
+
+            if initial_state != self.csc.default_initial_state:
+                # Check all expected summary states expect the final state.
+                # That is omitted for backwards compatibility.
+                expected_states = get_expected_summary_states(
+                    initial_state=self.csc.default_initial_state,
+                    final_state=initial_state,
+                )[:-1]
+                for state in expected_states:
+                    await self.assert_next_summary_state(state)
+
             yield
         finally:
-            await self.remote.close()
-            await self.csc.close()
+            if self.remote is not None:
+                await self.remote.close()
+            if self.csc is not None:
+                await self.csc.close()
 
     async def assert_next_summary_state(
         self, state, flush=False, timeout=STD_TIMEOUT, remote=None
@@ -156,11 +178,11 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
         ----------
         state : `lsst.ts.salobj.State` or `int`
             Desired summary state.
-        flush : `bool` (optional)
+        flush : `bool`, optional
             Flush the read queue before waiting?
-        timeout : `float` (optional)
+        timeout : `float`, optional
             Time limit for getting the data sample (sec).
-        remote : `Remote` (optional)
+        remote : `Remote`, optional
             Remote to use; ``self.remote`` if None.
         """
         if remote is None:
@@ -182,9 +204,9 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
         ----------
         topic : `topics.ReadTopic`
             Topic to read, e.g. ``remote.evt_logMessage``.
-        flush : `bool` (optional)
+        flush : `bool`, optional
             Flush the read queue before waiting?
-        timeout : `double` (optional)
+        timeout : `double`, optional
             Time limit for getting the data sample (sec).
         kwargs : `dict`
             Dict of field_name: expected_value
@@ -200,6 +222,11 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
             read_value = getattr(data, field_name, None)
             if read_value is None:
                 self.fail(f"No such field {field_name} in topic {topic}")
+            if isinstance(expected_value, enum.IntEnum):
+                try:
+                    read_value = type(expected_value)(read_value)
+                except Exception:
+                    pass
             self.assertEqual(
                 read_value,
                 expected_value,
@@ -212,7 +239,9 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
         name,
         index,
         exe_name,
-        initial_state=sal_enums.State.STANDBY,
+        default_initial_state=sal_enums.State.STANDBY,
+        initial_state=None,
+        settings_to_apply=None,
         cmdline_args=(),
     ):
         """Test running the CSC command line script.
@@ -225,38 +254,77 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
             SAL index of component.
         exe_name : `str`
             Name of executable, e.g. "run_rotator.py"
-        initial_state : `lsst.ts.salobj.State` or `int` (optional)
-            The expected initial state of the CSC.
+        default_initial_state : `lsst.ts.salobj.State`, optional
+            The default initial state of the CSC.
+            Ignored unless `initial_state` is None.
+        initial_state : `lsst.ts.salobj.State` or `int` or `None`, optional
+            The desired initial state of the CSC; used to specify
+            the ``--state`` command-argument.
+        settings_to_apply : `str` or `None`, optional
+            Value for the ``--settings`` command-line argument.
+            Only relevant if ``initial_state`` is one of
+            `salobj.State.DISABLED` or `salobj.State.ENABLED`.
         cmdline_args : `List` [`str`]
             Additional command-line arguments, such as "--simulate".
         """
-        testutils.set_random_lsst_dds_domain()
+        testutils.set_random_lsst_dds_partition_prefix()
         exe_path = shutil.which(exe_name)
         if exe_path is None:
             self.fail(
                 f"Could not find bin script {exe_name}; did you setup or install this package?"
             )
 
-        if index in (None, 0):
-            process = await asyncio.create_subprocess_exec(exe_name, *cmdline_args)
+        args = [exe_name]
+        if index not in (None, 0):
+            args += [str(index)]
+        if initial_state is None:
+            expected_states = [default_initial_state]
         else:
-            process = await asyncio.create_subprocess_exec(
-                exe_name, str(index), *cmdline_args
+            args += ["--state", initial_state.name.lower()]
+            expected_states = get_expected_summary_states(
+                initial_state=default_initial_state, final_state=initial_state,
             )
-        try:
-            async with Domain() as domain, Remote(
-                domain=domain, name=name, index=index
-            ) as remote:
-                summaryState_data = await remote.evt_summaryState.next(
-                    flush=False, timeout=60
-                )
-                self.assertEqual(summaryState_data.summaryState, initial_state)
+        if settings_to_apply is not None:
+            args += ["--settings", settings_to_apply]
+        args += cmdline_args
 
-        finally:
-            process.terminate()
+        async with Domain() as domain, Remote(
+            domain=domain, name=name, index=index
+        ) as self.remote:
+            print("check_bin_script running:", " ".join(args))
+            process = await asyncio.create_subprocess_exec(
+                *args, stderr=subprocess.PIPE,
+            )
+            try:
+                for state in expected_states:
+                    await self.assert_next_summary_state(state, timeout=30)
+                if settings_to_apply is not None and settings_to_apply.endswith(
+                    ".yaml"
+                ):
+                    # The string settings_to_apply is a file name, and so
+                    # should appear in evt_settingsApplied.settingsVersion
+                    data = await self.remote.evt_settingsApplied.next(
+                        flush=False, timeout=STD_TIMEOUT
+                    )
+                    self.assertTrue(data.settingsVersion.startswith(settings_to_apply))
+            finally:
+                if process.returncode is None:
+                    process.terminate()
+                    await asyncio.wait_for(process.wait(), timeout=STD_TIMEOUT)
+                else:
+                    print("Warning: suprocess has already quit.")
+                    try:
+                        data = await process.stderr.read()
+                        print("Subprocess stderr: ", data.decode())
+                    except Exception as e:
+                        print(f"Could not read subprocess stderr: {e}")
 
     async def check_standard_state_transitions(
-        self, enabled_commands, skip_commands=None, timeout=STD_TIMEOUT
+        self,
+        enabled_commands,
+        skip_commands=None,
+        settingsToApply="",
+        timeout=STD_TIMEOUT,
     ):
         """Test standard CSC state transitions.
 
@@ -266,9 +334,12 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
             List of CSC-specific commands that are valid in the enabled state.
             Need not include the standard commands, which are "disable"
             and "setLogLevel" (which is valid in any state).
-        skip_commands : `List` [`str`] or `None`
+        skip_commands : `List` [`str`] or `None`, optional
             List of commands to skip.
-        timeout : `float`
+        settingsToApply : `str`, optional
+            Value for the ``settingsToApply`` argument for the ``start``
+            command.
+        timeout : `float`, optional
             Time limit for state transition commands (seconds).
 
         Notes
@@ -287,15 +358,19 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
         self.assertEqual(self.csc.summary_state, sal_enums.State.STANDBY)
         await self.assert_next_summary_state(sal_enums.State.STANDBY)
         await self.check_bad_commands(
-            good_commands=("start", "exitControl", "setLogLevel") + skip_commands
+            good_commands=("start", "exitControl", "setAuthList", "setLogLevel")
+            + skip_commands
         )
 
         # Send start; new state is DISABLED.
-        await self.remote.cmd_start.start(timeout=timeout)
+        await self.remote.cmd_start.set_start(
+            settingsToApply=settingsToApply, timeout=timeout
+        )
         self.assertEqual(self.csc.summary_state, sal_enums.State.DISABLED)
         await self.assert_next_summary_state(sal_enums.State.DISABLED)
         await self.check_bad_commands(
-            good_commands=("enable", "standby", "setLogLevel") + skip_commands
+            good_commands=("enable", "standby", "setAuthList", "setLogLevel")
+            + skip_commands
         )
 
         # Send enable; new state is ENABLED.
@@ -303,7 +378,9 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
         self.assertEqual(self.csc.summary_state, sal_enums.State.ENABLED)
         await self.assert_next_summary_state(sal_enums.State.ENABLED)
         all_enabled_commands = tuple(
-            sorted(set(("disable", "setLogLevel")) | set(enabled_commands))
+            sorted(
+                set(("disable", "setAuthList", "setLogLevel")) | set(enabled_commands)
+            )
         )
         await self.check_bad_commands(
             good_commands=all_enabled_commands + skip_commands
@@ -329,9 +406,9 @@ class BaseCscTestCase(metaclass=abc.ABCMeta):
 
         Parameters
         ----------
-        bad_commands : `List`[`str`] or `None` (optional)
+        bad_commands : `List`[`str`] or `None`, optional
             Names of bad commands to try, or None for all commands.
-        good_commands : `List`[`str`] or `None` (optional)
+        good_commands : `List`[`str`] or `None`, optional
             Names of good commands to skip, or None to skip none.
 
         Notes
