@@ -23,16 +23,17 @@ __all__ = ["stream_as_generator", "CscCommander"]
 
 import argparse
 import asyncio
+import collections
 import functools
 import sys
+import warnings
 
 from . import domain
 from . import remote
 from . import sal_enums
 from . import csc_utils
 
-# Timeout for fast operations (seconds)
-STD_TIMEOUT = 10
+STD_TIMEOUT = 60
 
 
 async def stream_as_generator(stream, encoding="utf-8"):
@@ -74,13 +75,20 @@ async def stream_as_generator(stream, encoding="utf-8"):
         yield line
 
 
-def round_any(value, digits=4):
+def round_any(value, digits=2):
     """Round any value to the specified number of digits.
 
     This is a no-op for int and str values.
+
+    Notes
+    -----
+    This is designed for use with DDS samples. As such, it expects
+    array values to be of type `list`.
     """
     if isinstance(value, float):
         return round(value, digits)
+    elif value and isinstance(value, list) and isinstance(value[0], float):
+        return [round(item, digits) for item in value]
     return value
 
 
@@ -91,16 +99,28 @@ class CscCommander:
     ----------
     name : `str`
         SAL component name of CSC.
-    index : `int` (optional)
+    index : `int`, optional
         SAL index of CSC.
-    exclude : `List` [`str`] or `None` (optional)
+    enable : `bool`, optional
+        Enable the CSC (when the commander starts up)?
+        Note: `amain` always supplies this argument.
+    exclude : `List` [`str`] or `None`, optional
         Names of telemetry or event topics to not print.
         If `None` or empty then no topics are excluded.
-    fields_to_ignore : `List` [`str`] (optional)
-        SAL topic fields to ignore when specifying command parameters,
+    exclude_commands : `List` [`str`], optional
+        Names of commands to ignore. Typically this should include:
+
+        * "abort" and "setValue", two generic commands that few CSCs implement.
+        * "enterControl" unless your CSC is "externally commandable",
+          meaning it can run in the OFFLINE state.
+
+        The default is to exclude nothing, for backwards compatibility.
+    fields_to_ignore : `List` [`str`], optional
+        SAL topic fields names to ignore when specifying command parameters,
         and when printing events and telemetry.
-    enable : `bool`
-        Enable the CSC (when the commander starts up)?
+    telemetry_fields_to_not_compare : `List` [`str`], optional
+        Telemetry field names to ignore when checking if a telemetry topic has
+        changed enough to print the latest sample. These fields are printed.
 
     Attributes
     ----------
@@ -124,65 +144,125 @@ class CscCommander:
     that runs a commander, consider picking a name that is obscure
     and not easily confused with the script that runs the CSC.
 
+    Make a subclass of `CscCommander` if you want to add extra commands
+    or provide special handling for some commands, events or telemetry.
     Subclasses may provide overrides as follows:
 
-    * ``do_<command_name>`` override handling a standard command
-      (one defined in the XML), or add a new command.
-      The method receives one argument: a list of arguments, each a string.
-      You must provide such a method for any standard command that takes
-      array arguments. It can also be useful for adding a custom command,
-      such as a command to execute a tracking sequence.
-      If you define a do_<command_name> method then also add an entry
-      to ``help_dict``, where: the key is the command name
-      and the value should be a brief (preferably only one line) help string
-      that lists the arguments first and possibly a brief description after.
-    * evt_<event_name>_callback overrides handling data for the specified
-      event (usually this just mean printing the data).
-      It receives one argument: the DDS sample.
-    * tel_<event_name>_callback overrides handling data for the specified
-      telemetry topic (usually this just involves printing the data).
-      It receives one argument: the DDS sample.
-      This can be useful if the default is too chatty.
+    * To hide unwanted commands: delete them from ``self.command_dict``
+      in your constructor. Most CSCs should hide the "abort", "enterControl"
+      and "setValue" commands, as follows::
+
+        for command_to_ignore in ("abort", "enterControl", "setValue"):
+            del self.command_dict[command_to_ignore]
+
+    * To override handling of a standard command (one defined in the XML):
+      define a ``do_<command_name>`` method.
+      The method receives one argument: a list of str arguments.
+      You should provide a custom handler for, or hide, any command with
+      array parameters, because the command parser only accepts scalars.
+
+    * To add an additional command (one not defined in the XML):
+
+        * Define a  ``do_<command_name>`` to handle the command.
+          The method receives one argument: a list of str arguments.
+
+        * Add an entry to ``help_dict``. The key is the command name
+          and the value is a brief (preferably only one line) help string
+          that lists the arguments first, and a brief description after.
+          Here is an example::
+
+            self.help_dict["sine"] = "start_position amplitude "
+            "# track one cycle of a sine wave",
+
+    * To override handling of an event or telemetry topic: define method
+      ``evt_<event_name>_callback`` or ``tel_<event_name>_callback``,
+      respectively.  It receives one argument: the DDS sample.
+      This can be especially useful if the default behavior is too chatty
+      for one or more telemetry topics.
 
     I have not found a way to write a unit test for this class.
     I tried running a commander in a subprocess but could not figure out
     how to send multiple commands (the ``suprocess.communicate``
     method only allows sending one item of data).
     Instead I suggest manually running it to control the Test CSC.
+
+    Examples
+    --------
+
+    You can do a lot with just the base class. This command-line script
+    will control the Test CSC pretty well::
+
+        import asyncio
+        from lsst.ts import salobj
+        asyncio.run(salobj.CscCommander.amain(name="Test", index=True))
+
+    However, this will claim to support some generic commands that the
+    Test CSC does not support ( "abort", "enterControl", and "setValue")
+    and it mishandles the "setArrays" command.
+    See `TestCscCommander` for a version that fixes these deficiencies.
+    `TestCscCommander` is run as follows::
+
+        import asyncio
+        from lsst.ts import salobj
+        asyncio.run(salobj.TestCscCommander.amain(index=True))
+
+    For an example with extra commands and special telemetry handling,
+    see ``RotatorCommander`` in the ts_rotator package.
+
+    **Unit Testing**
+
+    To unit test a commander:
+
+    * Set ``commander.testing = True``.
+      This appends all output messages to output queue
+      ``commander.output_queue``, a `collections.deque`.
+      The messages are also printed to ``stdout``, as usual.
+    * Call ``commander.run_command`` to run a command.
+    * Check the output using the output queue.
+    * Clear the output queue with ``commander.output_queue.clear()``
+      whenever you like, e.g. before running a command whose output
+      you want to check.
     """
 
     def __init__(
         self,
         name,
         index=0,
-        exclude=None,
-        fields_to_ignore=("ignored", "value", "priority"),
         enable=False,
+        exclude=None,
+        exclude_commands=(),
+        fields_to_ignore=("ignored", "value", "priority"),
+        telemetry_fields_to_not_compare=("timestamp",),
     ):
         self.domain = domain.Domain()
         self.remote = remote.Remote(
             domain=self.domain, name=name, index=index, exclude=exclude
         )
-        self.fields_to_ignore = set(fields_to_ignore)
+        self.fields_to_ignore = frozenset(fields_to_ignore)
+        self.telemetry_fields_to_not_compare = frozenset(
+            telemetry_fields_to_not_compare
+        )
         self.tasks = set()
         self.help_dict = dict()
         self.enable = enable
+        self.testing = False
+        self.output_queue = collections.deque()
 
         for name in self.remote.salinfo.event_names:
             if name == "heartbeat":
                 continue
-            topic_name = f"evt_{name}"
-            topic = getattr(self.remote, topic_name)
-            callback = getattr(self, f"{topic_name}_callback", None)
+            topic_attr_name = f"evt_{name}"
+            topic = getattr(self.remote, topic_attr_name)
+            callback = getattr(self, f"{topic_attr_name}_callback", None)
             if callback is None:
                 callback = functools.partial(self.event_callback, name=name)
             setattr(topic, "callback", callback)
 
         for name in self.remote.salinfo.telemetry_names:
-            topic_name = f"tel_{name}"
-            setattr(self, f"previous_{topic_name}", None)
-            topic = getattr(self.remote, topic_name)
-            callback = getattr(self, f"{topic_name}_callback", None)
+            topic_attr_name = f"tel_{name}"
+            setattr(self, f"previous_{topic_attr_name}", None)
+            topic = getattr(self.remote, topic_attr_name)
+            callback = getattr(self, f"{topic_attr_name}_callback", None)
             if callback is None:
                 callback = functools.partial(self.telemetry_callback, name=name)
             setattr(topic, "callback", callback)
@@ -191,13 +271,14 @@ class CscCommander:
         self.command_dict = {
             name: getattr(self.remote, f"cmd_{name}")
             for name in self.remote.salinfo.command_names
+            if name not in frozenset(exclude_commands)
         }
 
     def print_help(self):
         """Print help.
         """
         command_help = "\n".join(self.get_commands_help())
-        print(
+        self.output(
             f"""Send commands to the {self.remote.salinfo.name} CSC and print events and telemetry.
 
 CSC Commands:
@@ -216,6 +297,19 @@ help  # print this help
         await self.remote.close()
         await self.domain.close()
 
+    def output(self, str):
+        """Print a string to output, appending a final newline.
+
+        Please call this instead of print to support unit tests.
+
+        If ``self.testing`` is True then append ``str`` to
+        ``self.output_queue`` instead of printing it.
+        Use this mode for unit testing a CSC commander.
+        """
+        if self.testing:
+            self.output_queue.append(str)
+        print(str)
+
     def format_item(self, key, value):
         """Format one event or telemetry field for printing.
         """
@@ -224,12 +318,17 @@ help  # print this help
         return f"{key}={value}"
 
     def format_data(self, data):
-        """Format an event or telemetry sample for printing.
+        """Format the public fields of an event or telemetry sample,
+        for printing.
         """
-        return ", ".join(
-            self.format_item(key, value)
-            for key, value in self.get_public_data(data).items()
-        )
+        return self.format_dict(self.get_public_data(data))
+
+    def format_dict(self, data):
+        """Format a dict for printing.
+
+        Unlike format_data, this requires a dict and formats *all* fields.
+        """
+        return ", ".join(self.format_item(key, value) for key, value in data.items())
 
     def field_is_public(self, name):
         """Return True if the specified field name is public,
@@ -244,7 +343,7 @@ help  # print this help
         return True
 
     def get_public_data(self, data):
-        """Return a dict of field_name: value for public fields.
+        """Get a dict of field_name: value for public fields of a DDS sample.
 
         Parameters
         ----------
@@ -257,44 +356,56 @@ help  # print this help
             if self.field_is_public(key)
         )
 
-    def get_rounded_public_fields(self, data):
-        """Get the public fields for a sample, with float values rounded.
+    def get_rounded_public_data(self, data, digits=2):
+        """Get a dict of field_name: value for public fields of a DDS sample
+        with float values rounded.
         """
         return {
-            key: round_any(value)
+            key: round_any(value, digits=digits)
             for key, value in data.get_vars().items()
             if self.field_is_public(key)
         }
 
+    def get_rounded_public_fields(self, data, digits=2):
+        """Deprecated version of get_rounded_public_data.
+        """
+        warnings.warn("Use get_rounded_public_data instead", DeprecationWarning)
+        return self.get_rounded_public_data(data=data, digits=digits)
+
     def event_callback(self, data, name):
         """Generic callback for events.
 
         You may provide evt_<event_name> methods to override printing
         of specific events.
         """
-        print(f"{data.private_sndStamp:0.3f}: {name}: {self.format_data(data)}")
+        self.output(f"{data.private_sndStamp:0.3f}: {name}: {self.format_data(data)}")
 
     def evt_summaryState_callback(self, data):
         try:
             state = sal_enums.State(data.summaryState)
         except Exception:
             state = f"{data.summaryState} (not a known state!)"
-        print(f"{data.private_sndStamp:0.3f}: summaryState: summaryState={state!r}")
+        self.output(
+            f"{data.private_sndStamp:0.3f}: summaryState: summaryState={state!r}"
+        )
 
-    def telemetry_callback(self, data, name):
+    def telemetry_callback(self, data, name, digits=2):
         """Generic callback for telemetry.
 
         You may provide tel_<telemetry_name> methods to override printing
         of specific telemetry topics.
         """
         prev_value_name = f"previous_tel_{name}"
-        public_fields = self.get_rounded_public_fields(data)
-        if public_fields != getattr(self, prev_value_name):
-            setattr(self, prev_value_name, public_fields)
-            formatted_data = ", ".join(
-                f"{key}={value}" for key, value in public_fields.items()
-            )
-            print(f"{data.private_sndStamp:0.3f}: {name}: {formatted_data}")
+        public_dict = self.get_rounded_public_data(data, digits=digits)
+        trimmed_dict = {
+            name: value
+            for name, value in public_dict.items()
+            if name not in self.telemetry_fields_to_not_compare
+        }
+        if trimmed_dict != getattr(self, prev_value_name):
+            setattr(self, prev_value_name, trimmed_dict)
+            formatted_data = self.format_dict(public_dict)
+            self.output(f"{data.private_sndStamp:0.3f}: {name}: {formatted_data}")
 
     def check_arguments(self, args, *names):
         """Check that the required arguments are provided,
@@ -358,8 +469,8 @@ help  # print this help
             if field_names_str is None:
                 command_topic = self.command_dict[command_name]
                 sample = command_topic.DataType()
-                public_data = self.get_public_data(sample)
-                field_names_str = " ".join(public_data.keys())
+                public_dict = self.get_public_data(sample)
+                field_names_str = " ".join(public_dict.keys())
             help_strings.append(f"{command_name} {field_names_str}")
 
         other_command_names = sorted(
@@ -404,32 +515,64 @@ help  # print this help
             kwargs[name] = type(default_value)(str_value)
         await command.set_start(**kwargs)
 
-    async def run_coroutine(self, coro, command_name):
-        """Run a coroutine and manage the task in self.tasks.
-
-        This allows the task to be cancelled in `close`.
+    async def run_command(self, cmd):
+        """Run the specified command string and wait for it to finish.
 
         Parameters
         ----------
-        coro : awaitable
-            Coroutine or other awaitable.
+        cmd : `str`
+            Command string (command name and arguments).
+            Note: does not handle the "exit" command.
         """
-        task = asyncio.create_task(coro)
+        tokens = cmd.split()
+        command_name = tokens[0]
+        args = tokens[1:]
+        command_method = getattr(self, f"do_{command_name}", None)
+        coro = None
+        if command_name == "help":
+            self.print_help()
+        elif command_method is not None:
+            coro = command_method(args)
+        elif command_name in self.command_dict:
+            coro = self.run_command_topic(command_name, args)
+        else:
+            self.output(f"Unrecognized command: {command_name}")
+
+        if coro is None:
+            return
+        await coro
+
+    async def _run_command_and_output(self, cmd):
+        """Execute a command and wait for it to finish. Output the result.
+
+        A wrapper around `run_command` that adds a task to self.tasks
+        and catches and outputs exceptions. For use by the interactive
+        command loop.
+
+        Parameters
+        ----------
+        cmd : `str`
+            Command string (command name and arguments).
+            Note: does not handle the "exit" command.
+        """
+        task = asyncio.create_task(self.run_command(cmd))
         self.tasks.add(task)
         try:
             await task
-            print(f"Finished command {command_name}")
+            command_name = cmd.split()[0]
+            self.output(f"Finished command {command_name}")
         except Exception as e:
-            print(f"Command {command_name} failed: {e}")
-        self.tasks.discard(task)
+            self.output(str(e))
+        finally:
+            self.tasks.remove(task)
 
     async def start(self):
         """Start asynchonous processes.
         """
-        print(f"Waiting for {self.remote.salinfo.name_index} to start.")
+        self.output(f"Waiting for {self.remote.salinfo.name_index} to start.")
         await self.remote.start_task
         if self.enable:
-            print(f"Enabling {self.remote.salinfo.name_index}")
+            self.output(f"Enabling {self.remote.salinfo.name_index}")
             # Temporarily remove the ``evt_summaryState`` callback
             # so the `set_summary_state` function can read the topic.
             summary_state_callback = self.remote.evt_summaryState.callback
@@ -443,40 +586,42 @@ help  # print this help
                 self.remote.evt_summaryState.callback = summary_state_callback
 
     @classmethod
-    async def amain(cls, **kwargs):
+    async def amain(cls, *, index, **kwargs):
         """Construct the commander and run it.
 
         Parse the command line to construct the commander,
         then parse and execute commands until the ``exit`` is seen.
+
+        Parameters
+        ----------
+        index : `int`, `True`, `False` or `None`
+            If the CSC is indexed: specify `True` make index a required
+            command line argument, or specify a non-zero `int` to use
+            that index.
+            If the CSC is not indexed: specify `None` or 0.
+        **kwargs : `dict`, optional
+            Additional keyword arguments for your CSC's constructor.
         """
-        self = cls.make_from_cmd_line(**kwargs)
+        self = cls.make_from_cmd_line(index=index, **kwargs)
         try:
             await self.start()
 
             self.print_help()
             async for line in stream_as_generator(sys.stdin):
+                # Purge done tasks
+                self.tasks = {task for task in self.tasks if not task.done()}
+
+                # Execute the new command
                 line = line.strip()
                 # Strip trailing comment, if any.
                 if "#" in line:
                     line = line.split("#", maxsplit=1)[0].strip()
                 if not line:
                     continue
-                tokens = line.split()
-                command_name = tokens[0]
-                args = tokens[1:]
-                command_method = getattr(self, f"do_{command_name}", None)
-                if command_name == "exit":
+                if line == "exit":
                     break
-                elif command_name == "help":
-                    self.print_help()
-                elif command_method is not None:
-                    coro = command_method(args)
-                    asyncio.create_task(self.run_coroutine(coro, command_name))
-                elif command_name in self.command_dict:
-                    coro = self.run_command_topic(command_name, args)
-                    asyncio.create_task(self.run_coroutine(coro, command_name))
-                else:
-                    print(f"Unrecognized command: {command_name}")
+                task = asyncio.create_task(self._run_command_and_output(line))
+                self.tasks.add(task)
         finally:
             await self.close()
 
@@ -527,10 +672,8 @@ help  # print this help
             a required command line argument,
             or specify a non-zero `int` to use that index.
             If the SAL component is not indexed: specify `None` or 0.
-        **kwargs : `dict` (optional)
+        **kwargs : `dict`, optional
             Additional keyword arguments for your class's constructor.
-            If any arguments match those from the command line
-            the command line values will be used.
 
         Returns
         -------
