@@ -29,6 +29,7 @@ import numpy as np
 
 from .base_csc import State
 from .configurable_csc import ConfigurableCsc
+from . import __version__
 
 
 class TestCsc(ConfigurableCsc):
@@ -54,24 +55,23 @@ class TestCsc(ConfigurableCsc):
     index : `int`
         Index of Test component; each unit test method
         should use a different index.
-    config_dir : `str` (optional)
+    config_dir : `str`, optional
         Path to configuration files.
-    initial_state : `salobj.State` (optional)
+    initial_state : `salobj.State`, optional
         The initial state of the CSC. Typically one of:
 
         * `salobj.State.ENABLED` if you want the CSC immediately usable.
         * `salobj.State.STANDBY` if you want full emulation of a CSC.
-    simulation_mode : `int` (optional)
+    settings_to_apply : `str`, optional
+        Settings to apply if ``initial_state`` is `State.DISABLED`
+        or `State.ENABLED`.
+    simulation_mode : `int`, optional
         Simulation mode. The only allowed value is 0.
-    initial_simulation_mode : `int` (optional)
-        A deprecated synonym for ``simulation_mode``.
 
     Raises
     ------
     ValueError
-        If ``config_dir`` is not a directory,
-        ``initial_state`` is invalid, or
-        ``simulation_mode`` and ``initial_simulation_mode`` are both nonzero.
+        If ``config_dir`` is not a directory or ``initial_state`` is invalid.
     salobj.ExpectedError
         If ``simulation_mode`` is invalid.
         Note: you will only see this error if you await `start_task`.
@@ -95,6 +95,9 @@ class TestCsc(ConfigurableCsc):
     * 1: the fault command was executed
     """
 
+    enable_cmdline_state = True
+    valid_simulation_modes = [0]
+    version = __version__
     __test__ = False  # stop pytest from warning that this is not a test
 
     def __init__(
@@ -102,8 +105,8 @@ class TestCsc(ConfigurableCsc):
         index,
         config_dir=None,
         initial_state=State.STANDBY,
+        settings_to_apply="",
         simulation_mode=0,
-        initial_simulation_mode=0,
     ):
         schema_path = (
             pathlib.Path(__file__).resolve().parents[4] / "schema" / "Test.yaml"
@@ -114,8 +117,8 @@ class TestCsc(ConfigurableCsc):
             config_dir=config_dir,
             index=index,
             initial_state=initial_state,
+            settings_to_apply=settings_to_apply,
             simulation_mode=simulation_mode,
-            initial_simulation_mode=initial_simulation_mode,
         )
         self.cmd_wait.allow_multiple_callbacks = True
         self.config = None
@@ -164,13 +167,16 @@ class TestCsc(ConfigurableCsc):
         self.fault(code=1, report="executing the fault command")
 
     async def do_wait(self, data):
-        """Execute the wait command.
+        """Execute the wait command by waiting for the specified duration.
 
-        Wait for the specified time and then acknowledge the command
-        using the specified ack code.
+        If duration is negative then wait for abs(duration) but do not
+        acknowledge the command as "in progress". This is useful for
+        testing command timeout.
         """
         self.assert_enabled()
-        await asyncio.sleep(data.duration)
+        if data.duration >= 0:
+            self.cmd_wait.ack_in_progress(data, timeout=data.duration)
+        await asyncio.sleep(abs(data.duration))
 
     @property
     def field_type(self):
