@@ -24,7 +24,9 @@ __all__ = [
     "assertRaisesAckError",
     "assertRaisesAckTimeoutError",
     "assert_black_formatted",
-    "set_random_lsst_dds_domain",
+    "set_random_lsst_dds_domain",  # Deprecated
+    "set_random_lsst_dds_partition_prefix",
+    "modify_environ",
 ]
 
 import contextlib
@@ -33,6 +35,7 @@ import random
 import socket
 import subprocess
 import time
+import warnings
 
 import astropy.coordinates
 import astropy.units as u
@@ -69,12 +72,12 @@ def assertRaisesAckError(ack=None, error=None, result_contains=None):
 
     Parameters
     ----------
-    ack : `int` (optional)
+    ack : `int`, optional
         Ack code, almost always a `SalRetCode` ``CMD_<x>`` constant.
         If None then the ack code is not checked.
-    error : `int` (optional)
+    error : `int`, optional
         Error code. If None then the error value is not checked.
-    result_contains : `str` (optional)
+    result_contains : `str`, optional
         If not None then the result value must contain this string.
     """
     try:
@@ -97,10 +100,10 @@ def assertRaisesAckTimeoutError(ack=None, error=None):
 
     Parameters
     ----------
-    ack : `int` (optional)
+    ack : `int`, optional
         Ack code of the last ack seen, almost always a `SalRetCode`
         ``CMD_<x>`` constant. If None then the ack code is not checked.
-    error : `int` (optional)
+    error : `int`, optional
         Error code. If None then the error value is not checked.
     """
     try:
@@ -113,28 +116,39 @@ def assertRaisesAckTimeoutError(ack=None, error=None):
             raise AssertionError(f"ackcmd.error={e.ackcmd.error} instead of {error}")
 
 
-def assert_black_formatted(dir):
-    """Assert that all Python files in a directory (at any depth)
-    are formatted with black.
+def assert_black_formatted(dirpath):
+    """Assert that all Python files in a directory, or any subdirectory
+    of that directory, are formatted with black.
 
-    Here is how to call this from a unit test:
+    To call this from a unit test (see ``tests/test_black.py``)::
 
-        assert_black_formatted(pathlib.Path(__file__).parents[1])
+        salobj.assert_black_formatted(pathlib.Path(__file__).parents[1])
+
+    Parameters
+    ----------
+    dirpath : `pathlib.Path` or `str`
+        Path to root directory; typically this will be the root
+        directory of a package, such as ts_salobj.
 
     Raises
     ------
     AssertionError
         If any files are not formatted with ``black``.
+
+    Notes
+    -----
+    For this test to exclude ``version.py``, list it in ``.gitignore``
+    as ``version.py``, rather than by the full path
+    (e.g. not `python/lsst/ts/salobj/version.py`).
+    This may be a bug in black 19.10b0.
     """
-    result = subprocess.run(
-        ["black", "--check", str(dir), "--exclude", "version.py"], capture_output=True
-    )
+    result = subprocess.run(["black", "--check", str(dirpath)], capture_output=True,)
     if result.returncode != 0:
         raise AssertionError(result.stderr)
 
 
-def set_random_lsst_dds_domain():
-    """Set a random value for environment variable LSST_DDS_DOMAIN
+def set_random_lsst_dds_partition_prefix():
+    """Set a random value for environment variable LSST_DDS_PARTITION_PREFIX
 
     Call this for each unit test method that uses SAL message passing,
     in order to avoid collisions with other tests. Note that pytest
@@ -149,4 +163,55 @@ def set_random_lsst_dds_domain():
     hostname = socket.gethostname()
     curr_time = time.time()
     random_int = random.randint(0, 999999)
-    os.environ["LSST_DDS_DOMAIN"] = f"Test-{hostname}-{curr_time}-{random_int}"
+    os.environ[
+        "LSST_DDS_PARTITION_PREFIX"
+    ] = f"Test-{hostname}-{curr_time}-{random_int}"
+
+
+def set_random_lsst_dds_domain():
+    """Deprecated version of `set_random_lsst_dds_partition_prefix`."""
+    warnings.warn(
+        "Use set_random_lsst_dds_partition_prefix instead", DeprecationWarning
+    )
+    set_random_lsst_dds_partition_prefix()
+
+
+@contextlib.contextmanager
+def modify_environ(**kwargs):
+    """Context manager to temporarily modify os.environ.
+
+    Parameters
+    ----------
+    kwargs : `dict` [`str`, `str` or `None`]
+        Environment variables to set or clear.
+        Each value must be one of:
+
+        * A string value to set the env variable.
+        * None to delete the env variable (if present).
+
+    Raises
+    ------
+    RuntimeError
+        If any value in kwargs is not of type `str`
+        or if a name appears in ``delete`` and kwargs.
+    """
+    bad_value_strs = [
+        f"{name}: {value!r}"
+        for name, value in kwargs.items()
+        if not isinstance(value, str) and value is not None
+    ]
+    if bad_value_strs:
+        raise RuntimeError(
+            "The following values are not of type str or None: "
+            + ", ".join(bad_value_strs)
+        )
+
+    original_values = os.environ.copy()
+    for name, value in kwargs.items():
+        if value is None:
+            os.environ.pop(name, None)
+        else:
+            os.environ[name] = value
+    yield
+    os.environ.clear()
+    os.environ.update(original_values)
