@@ -71,7 +71,7 @@ class BaseScriptTestCase(asynctest.TestCase):
     """
 
     def setUp(self):
-        salobj.set_random_lsst_dds_domain()
+        salobj.set_random_lsst_dds_partition_prefix()
         self.datadir = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
         self.index = next(index_gen)
 
@@ -85,13 +85,13 @@ class BaseScriptTestCase(asynctest.TestCase):
         ----------
         script : `ts.salobj.TestScript`
             A test script
-        log_level : `int` (optional)
+        log_level : `int`, optional
             Log level as a `logging` level,
             or 0 to leave the script's log level unchanged.
-        pause_checkpoint : `str` (optional)
+        pause_checkpoint : `str`, optional
             Checkpoint(s) at which to pause, as a regular expression.
             "" to not pause at any checkpoint; "*" to pause at all checkpoints.
-        stop_checkpoint : `str` (optional)
+        stop_checkpoint : `str`, optional
             Checkpoint(s) at which to stop, as a regular expression.
             "" to not stop at any checkpoint; "*" to stop at all checkpoints.
         kwargs : `dict`
@@ -161,12 +161,8 @@ class BaseScriptTestCase(asynctest.TestCase):
 
     async def test_non_configurable_script_empty_config(self):
         """Test configuring the script with no data. This should work.
-
-        Also test that creating the script sets the master priority
-        environment variable to "0".
         """
         async with NonConfigurableScript(index=self.index) as script:
-            self.assertEqual(os.environ[salobj.MASTER_PRIORITY_ENV_VAR], "0")
             data = script.cmd_configure.DataType()
             await script.do_configure(data)
             self.assertEqual(len(script.config.__dict__), 0)
@@ -181,6 +177,24 @@ class BaseScriptTestCase(asynctest.TestCase):
                 await script.do_configure(data)
             self.assertIsNone(script.config)
 
+    async def test_script_environ(self):
+        """Test that creating a script does not modify os.environ.
+
+        I would like to also test that a script has master priority 0
+        (which is done by temporarily setting env var
+        salobj.MASTER_PRIORITY_ENV_VAR to "0"),
+        but that information is not available.
+        """
+        for master_priority in ("21", None):
+            env_kwargs = {salobj.MASTER_PRIORITY_ENV_VAR: master_priority}
+            with salobj.modify_environ(**env_kwargs):
+                initial_environ = os.environ.copy()
+                async with NonConfigurableScript(index=self.index):
+                    self.assertEqual(os.environ, initial_environ)
+                # Test again when script is closed, just to be sure;
+                # closing a script should not modify the environment.
+                self.assertEqual(os.environ, initial_environ)
+
     async def test_setCheckpoints(self):
         async with salobj.TestScript(index=self.index) as script:
 
@@ -543,11 +557,7 @@ class BaseScriptTestCase(asynctest.TestCase):
                 async with salobj.Domain() as domain:
                     index = next(index_gen)
                     remote = salobj.Remote(
-                        domain=domain,
-                        name="Script",
-                        index=index,
-                        evt_max_history=0,
-                        tel_max_history=0,
+                        domain=domain, name="Script", index=index, evt_max_history=0,
                     )
                     await asyncio.wait_for(remote.start_task, timeout=STD_TIMEOUT)
 
@@ -639,57 +649,6 @@ class BaseScriptTestCase(asynctest.TestCase):
                 process.terminate()
                 warnings.warn("Killed a process that was not properly terminated")
 
-    async def test_script_deprecated_main_process(self):
-        """Test running a script using deprecated class method ``main``
-        """
-        script_path = os.path.join(self.datadir, "script_with_deprecated_main")
-
-        async with salobj.Domain() as domain:
-            index = next(index_gen)
-            remote = salobj.Remote(domain=domain, name="Script", index=index)
-            await asyncio.wait_for(remote.start_task, timeout=STD_TIMEOUT)
-
-            process = await asyncio.create_subprocess_exec(script_path, str(index))
-            try:
-                self.assertIsNone(process.returncode)
-
-                state = await remote.evt_state.next(flush=False, timeout=STD_TIMEOUT)
-                self.assertEqual(state.state, ScriptState.UNCONFIGURED)
-                self.assertEqual(state.groupId, "")
-
-                logLevel_data = await remote.evt_logLevel.next(
-                    flush=False, timeout=STD_TIMEOUT
-                )
-                self.assertEqual(logLevel_data.level, logging.INFO)
-
-                wait_time = 0.1
-                config = f"wait_time: {wait_time}"
-                await remote.cmd_configure.set_start(config=config, timeout=STD_TIMEOUT)
-                state = await remote.evt_state.next(flush=False, timeout=STD_TIMEOUT)
-                self.assertEqual(state.state, ScriptState.CONFIGURED)
-                self.assertEqual(state.groupId, "")
-
-                metadata = await remote.evt_metadata.next(
-                    flush=False, timeout=STD_TIMEOUT
-                )
-                self.assertEqual(metadata.duration, wait_time)
-
-                group_id = "a non-blank group ID"
-                await remote.cmd_setGroupId.set_start(
-                    groupId=group_id, timeout=STD_TIMEOUT
-                )
-                state = await remote.evt_state.next(flush=False, timeout=STD_TIMEOUT)
-                self.assertEqual(state.groupId, group_id)
-
-                await remote.cmd_run.start(timeout=STD_TIMEOUT)
-
-                await asyncio.wait_for(process.wait(), timeout=STD_TIMEOUT)
-                self.assertEqual(process.returncode, 0)
-            finally:
-                if process.returncode is None:
-                    process.terminate()
-                    warnings.warn("Killed a process that was not properly terminated")
-
 
 if __name__ == "__main__":
     unittest.main()
